@@ -1,4 +1,4 @@
-__version__ = '1.1.0'
+__version__ = '1.1.1'
 __all__ = ["cloud", "collPartner", "composition", "despoticError", \
            "dustProp", "emitter", "emitterData", "fetchLamda", \
            "lineProfLTE", "radiation", "refreshLamda"]
@@ -8,6 +8,10 @@ __all__ = ["cloud", "collPartner", "composition", "despoticError", \
 # directly, as despotic.cloud, rather than having to use the
 # cumbersome notation despotic.cloud.cloud or similar.
 from cloud import cloud
+try:
+    from cloud import run_cloud_gui
+except:
+    pass
 from collPartner import collPartner
 from composition import composition
 from dustProp import dustProp
