@@ -99,6 +99,7 @@ _out2bdy2 = np.array([10, 10, 10, 10, 8, 10, 10, 10, 10, 10, 10,
 ########################################################################
 # Set some default abundances
 ########################################################################
+_xHedefault = 0.1
 _xCdefault = 2.0e-4
 _xOdefault = 4.0e-4
 _xMdefault = 2.0e-7
@@ -179,7 +180,7 @@ class NL99(chemNetwork):
             self.cloud = None
 
             # Physical properties
-            self._xHe = 0.1
+            self._xHe = _xHedefault
             self._ionRate = 2.0e-17
             self._NH = small
             self._temp = small
@@ -215,6 +216,12 @@ class NL99(chemNetwork):
                 raise despoticError, "NL99 network only valid " + \
                     "for pure H2 composition"
 
+            # Sanity check: make sure cloud contains some He, since
+            # network will not function properly at He abundance of 0
+            if cloud.comp.xHe == 0.0:
+                raise despoticError, "NL99 network requires " + \
+                    "non-zero He abundance"
+
             # Set abundances
 
             # Make a case-insensitive version of the emitter list for
@@ -265,17 +272,17 @@ class NL99(chemNetwork):
             if xCtot < xC:
                 # Print warning if we're altering existing C+
                 # abundance.
-                if self.x[6] != 0.0:
+                if 'c' in emList:
                     print "Warning: input C abundance is " + \
-                        str(xC) + ", but total input C, C+, CO, " + \
+                        str(xC) + ", but total input C, C+, CHx, CO, " + \
                         "HCO+ abundance is " + str(xCtot) + \
-                        "; increasing xC+ to " + str(xC-xCtot)
-                self.x[6] = xC - xCtot
+                        "; increasing xC+ to " + str(self.x[6]+xC-xCtot)
+                self.x[6] += xC - xCtot
             elif xCtot > xC:
                 # Throw an error if input C abundance is smaller than
                 # what is accounted for in initial conditions
                 raise despoticError, "input C abundance is " + \
-                    str(xC) + ", but total input C, C+, CO, " + \
+                    str(xC) + ", but total input C, C+, CHx, CO, " + \
                     "HCO+ abundance is " + str(xCtot)
 
             # O
@@ -283,13 +290,40 @@ class NL99(chemNetwork):
                 self.x[8] = emList['o'].abundance
             elif info is None:
                 self.x[8] = _xOdefault - self.x[2] - self.x[4] - \
-                    self.x[6] - self.x[7]
+                    self.x[7]
             elif 'xO' in info:
                 self.x[8] = info['xO'] - self.x[2] - self.x[4] - \
-                    self.x[6] - self.x[7]
+                    self.x[7]
             else:
                 self.x[8] = _xOdefault - self.x[2] - self.x[4] - \
-                    self.x[6] - self.x[7]
+                    self.x[7]
+
+            # As with C, make sure all O is accounted for, and if not
+            # park the extra in OI
+            if info is None:
+                xO = _xOdefault
+            elif 'xC' in info:
+                xO = info['xO']
+            else:
+                xO = _xOdefault
+            xOtot = self.x[2] + self.x[4] + self.x[7] + self.x[8]
+            if xOtot < xO:
+                # Print warning if we're altering existing O
+                # abundance.
+                if 'o' in emList:
+                    print "Warning: input O abundance is " + \
+                        str(xO) + ", but total input O, OHx, CO, " + \
+                        "HCO+ abundance is " + str(xOtot) + \
+                        "; increasing xO to " + str(self.x[8]+xO-xOtot)
+                self.x[8] += xO - xOtot
+            elif xOtot > xO:
+                # Throw an error if input O abundance is smaller than
+                # what is accounted for in initial conditions
+                raise despoticError, "input C abundance is " + \
+                    str(xO) + ", but total input O, OHx, CO, " + \
+                    "HCO+ abundance is " + str(xOtot)
+
+
 
         # Initial electrons = metals + C+ + HCO+
         xeinit = self.xM + self.x[6] + self.x[7]
@@ -497,7 +531,7 @@ class NL99(chemNetwork):
         xdot[1] = self.ionRate
 
         # Photon reactions
-        ratecoef = self.chi/1.6*np.exp(-_avfac*self.AV)*_kph
+        ratecoef = 1.7*self.chi*np.exp(-_avfac*self.AV)*_kph
         rate = ratecoef*xgrow[_inph]
         # Apply CO line shielding factor
         rate[2] = rate[2] * fShield_CO_vDB(xgrow[4]*self.NH, self.NH/2.0) 
